/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

/* global $ */

if (!QfqNS) {
    var QfqNS = {};
}

(function (n) {
    'use strict';

    n.FileUpload = function (formSelector, targetUrl, sip) {
        this.formSelector = formSelector;
        this.targetUrl = targetUrl;
        this.sip = sip;

        // TODO: Seriously, I'd like to have an event system.
        this.fileUploadStartCallbacks = new n.Helper.FunctionList();
        this.fileUploadEndCallbacks = new n.Helper.FunctionList();
        this.fileUploadSuccessCallbacks = new n.Helper.FunctionList();
        this.fileUploadErrorCallbacks = new n.Helper.FunctionList();

        this.setupOnChangeHandler();
    };

    /**
     * @public
     * @param handler
     */
    n.FileUpload.prototype.addFileUploadStartHandler = function (handler) {
        this.fileUploadStartCallbacks.addFunction(handler);
    };

    /**
     * @public
     * @param handler
     */
    n.FileUpload.prototype.addFileUploadEndHandler = function (handler) {
        this.fileUploadEndCallbacks.addFunction(handler);
    };

    /**
     * @public
     * @param handler
     */
    n.FileUpload.prototype.addFileUploadSuccessHandler = function (handler) {
        this.fileUploadSuccessCallbacks.addFunction(handler);
    };

    /**
     * @public
     * @param handler
     */
    n.FileUpload.prototype.addFileUploadErrorHandler = function (handler) {
        this.fileUploadErrorCallbacks.addFunction(handler);
    };

    /**
     *
     * @private
     */
    n.FileUpload.prototype.setupOnChangeHandler = function () {
        $(this.formSelector + " :file").on('change', this.handleFileUpload.bind(this));
    };

    /**
     * @private
     * @param event
     */
    n.FileUpload.prototype.handleFileUpload = function (event) {
        this.fileUploadStartCallbacks.call(event);

        var data = this.prepareData(event.target);

        $.ajax({
            url: this.targetUrl,
            type: 'POST',
            data: data,
            cache: false,
            processData: false,
            contentType: false
        })
            .done(this.ajaxSuccessHandler.bind(this))
            .fail(this.ajaxErrorHandler.bind(this));
    };

    n.FileUpload.prototype.prepareData = function (htmlFileInput) {
        if (!htmlFileInput.hasAttribute("name")) {
            throw new Error("File input element requires 'name' attribute");
        }

        var fileInputName = htmlFileInput.getAttribute('name');

        var data = new FormData();
        $.each(htmlFileInput.files, function (key, value) {
            data.append(fileInputName + '_' + key, value);
        });

        data.append('s', this.sip);

        return data;
    };

    /**
     * @private
     * @param data
     * @param textStatus
     * @param jqXHR
     */

    n.FileUpload.prototype.ajaxSuccessHandler = function (data, textStatus, jqXHR) {
        this.fileUploadSuccessCallbacks.call(data, textStatus);
        this.fileUploadEndCallbacks.call(this);
    };

    /**
     * @private
     * @param jqXHR
     * @param textStatus
     * @param errorThrown
     */
    n.FileUpload.prototype.ajaxErrorHandler = function (jqXHR, textStatus, errorThrown) {
        this.fileUploadErrorCallbacks.call(textStatus, errorThrown);
        this.fileUploadEndCallbacks.call(this);
    };


})(QfqNS);
