/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

if (!QfqNS) {
    var QfqNS = {};
}

(function (n) {
    'use strict';


    n.PageState = function () {
        this.pageState = location.hash.slice(1);
        this.data = null;
        this.inPoppingHandler = false;
        this.userPopStateHandlers = new QfqNS.Helper.FunctionList();

        window.addEventListener("popstate", this.popStateHandler.bind(this));
    };
    /**
     *
     * @param event
     *
     * @private
     */
    n.PageState.prototype.popStateHandler = function (event) {

        QfqNS.Log.debug("Enter: PageState.popStateHandler()");

        this.inPoppingHandler = true;
        this.pageState = location.hash.slice(1);
        this.data = window.history.state;

        QfqNS.Log.debug("PageState.popStateHandler(): invoke user pop state handler(s)");

        this.userPopStateHandlers.call(this);

        this.inPoppingHandler = false;
        QfqNS.Log.debug("Exit: PageState.popStateHandler()");

    };

    n.PageState.prototype.getPageState = function () {
        return this.pageState;
    };

    n.PageState.prototype.getPageData = function () {
        return this.data;
    };

    n.PageState.prototype.setPageState = function (state, data) {
        if (state.startsWith('#')) {
            this.pageState = state.slice(1);
            window.history.replaceState(data, null, state);
        } else {
            this.pageState = state;
            window.history.replaceState(data, null, '#' + state);
        }
        this.data = data;
    };

    n.PageState.prototype.newPageState = function (state, data) {
        if (state.startsWith('#')) {
            this.pageState = state.slice(1);
            window.history.pushState(data, null, state);
        } else {
            this.pageState = state;
            window.history.pushState(data, null, '#' + state);
        }

        this.data = data;
    };

    n.PageState.prototype.addStateActivationHandler = function (handler) {
        this.userPopStateHandlers.addFunction(handler);
    };


})(QfqNS);